/**
 * Introduction
 *
 * @title ROV - Easy applications from C
 *
 * @copyright Jason Tribbeck / 7th ARM 1994-2001
 *
 * @authors Jason Tribbeck
 *
 * @file dir.c
 */

#include <stdio.h>
#include <string.h>

#include "kernel.h"
#include "dir.h"

typedef struct gbpb {
  int load;
  int exec;
  int size;
  int attr;
  int otype;
  int type;
  char name[64];
} gbpb;

int dir_scan(char * dirname, int flags, char * wild, dirscan_proc p)
{
  _kernel_swi_regs regs;
  char path[256];
  gbpb g;

  regs.r[0] = 12;
  regs.r[1] = (int)dirname;
  regs.r[2] = (int) & g;
  regs.r[4] = 0;
  regs.r[5] = sizeof(g);
  regs.r[6] = (int)wild;
  do {
    regs.r[3] = 1;
    if(_kernel_swi(0x2000c, & regs, & regs)) return 0;
    if(regs.r[3]) {
      switch(g.otype) {
      case 1 : // File
	if(flags & DIRSCAN_FILES) {
	  sprintf(path, "%s.%s", dirname, g.name);
	  p(path, g.type, g.size);
	}
	break;
      case 2 : // Directories
	if(flags & DIRSCAN_DIRS) {
	  sprintf(path, "%s.%s", dirname, g.name);
	  p(path, g.type, g.size);
	}
	if(flags & DIRSCAN_RECURSE) {
	  sprintf(path, "%s.%s", dirname, g.name);
	  dir_scan(path, flags, wild, p);
	}
	break;
      case 3 : // Image directories
	if(flags & DIRSCAN_IMAGEDIRS) {
	  sprintf(path, "%s.%s", dirname, g.name);
	  p(path, g.type, g.size);
	}
	if(flags & DIRSCAN_RECURSEIMAGEDIRS) {
	  sprintf(path, "%s.%s", dirname, g.name);
	  dir_scan(path, flags, wild, p);
	}
	break;
      }
    }
  } while(regs.r[3] == 1);
  return 1;
}

int dir_scan2(char * dirname, int flags, char * wild, dirscan2_proc p)
{
  _kernel_swi_regs regs;
  char path[256];
  gbpb g;

  regs.r[0] = 12;
  regs.r[1] = (int)dirname;
  regs.r[2] = (int) & g;
  regs.r[4] = 0;
  regs.r[5] = sizeof(g);
  regs.r[6] = (int)wild;
  do {
    regs.r[3] = 1;
    if(_kernel_swi(0x2000c, & regs, & regs)) return 0;
    if(regs.r[3]) {
      switch(g.otype) {
      case 1 : // File
	if(flags & DIRSCAN_FILES) {
	  sprintf(path, "%s.%s", dirname, g.name);
	  switch(p(path, g.type, g.size)) {
	  case DIRSCAN_BACKBYONE :
	    regs.r[4] -- ;
	    break;
	  case DIRSCAN_RESTARTDIR :
	    regs.r[4] = 0;
	    break;
	  }
	}
	break;
      case 2 : // Directories
	if(flags & DIRSCAN_RECURSE) {
	  sprintf(path, "%s.%s", dirname, g.name);
	  dir_scan2(path, flags, wild, p);
	}
	if(flags & DIRSCAN_DIRS) {
	  sprintf(path, "%s.%s", dirname, g.name);
	  switch(p(path, g.type, g.size)) {
	  case DIRSCAN_BACKBYONE :
	    regs.r[4] -- ;
	    break;
	  case DIRSCAN_RESTARTDIR :
	    regs.r[4] = 0;
	    break;
	  }
	}
	break;
      case 3 : // Image directories
	if(flags & DIRSCAN_RECURSEIMAGEDIRS) {
	  sprintf(path, "%s.%s", dirname, g.name);
	  dir_scan2(path, flags, wild, p);
	}
	if(flags & DIRSCAN_IMAGEDIRS) {
	  sprintf(path, "%s.%s", dirname, g.name);
	  switch(p(path, g.type, g.size)) {
	  case DIRSCAN_BACKBYONE :
	    regs.r[4] -- ;
	    break;
	  case DIRSCAN_RESTARTDIR :
	    regs.r[4] = 0;
	    break;
	  }
	}
	break;
      }
    }
  } while(regs.r[3] == 1);
  return 1;
}
