/**
 * Introduction
 *
 * @title ROV - Easy applications from C
 *
 * @copyright Jason Tribbeck / 7th ARM 1994-2001
 *
 * @authors Jason Tribbeck
 *
 * @file reghelp.c
 */

#include <stdio.h>
#include <string.h>

#include "main.h"
#include "canvas.h"
#include "icons.h"
#include "menu.h"
#include "extras.h"
#include "msg.h"

typedef struct regicon {
  char t[8];
  icon i;
  struct regicon * next;
} regicon;

typedef struct regcanv {
  char t[8];
  canvas c;
  struct regcanv * next;
} regcanv;

static regicon * regicons = 0;
static regcanv * regcanvs = 0;

static void registerhelp_canvass(canvas c)
{
  regcanv * r = regcanvs;

  while(r) {
    if(r -> c == c) {
      return_help(msg_lookup(r -> t));
      return;
    }
    r = r -> next;
  }
}

static void registerhelp_icons(canvas c, icon i)
{
  regicon * r = regicons;

  /* werr(FALSE, "RegHelp:c=%p, i=%p", c, i); */

  while(r) {
    if(r -> i == i) {
      return_help(msg_lookup(r -> t));
      return;
    }
    r = r -> next;
  }
  registerhelp_canvass(c);
}

void registerhelp_icon(icon i, char * t)
{
  regicon * r = rmalloc(sizeof(regicon));

  r -> next = regicons;
  r -> i = i;
  strcpy(r -> t, t);
  regicons = r;
  icon_addhandler(i, ICON_HELPHANDLER, registerhelp_icons);
}

void registerhelp_canvas(canvas c, char * t)
{
  regcanv * r = rmalloc(sizeof(regcanv));

  r -> next = regcanvs;
  r -> c = c;
  strcpy(r -> t, t);
  regcanvs = r;
  canvas_addhandler(c, CANVAS_HELPHANDLER, registerhelp_canvass);
}
